package dev.peerat.parser.java;

import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.function.Function;

import dev.peerat.parser.Token;

public class Annotation extends JavaElement{
	
	public static interface AnnotableBuffer{
		
		void addAnnotationBuffer(Annotation annotation);
		
		List<Annotation> getAnnotationBuffer();
		
	}
	
	private Token name;
	private Map<Token, Value> values;
	
	public Annotation(Token name){
		this.name = name;
	}
	
	public Annotation(Token name, Map<Token, Value> values){
		this(name);
		this.values =  values;
	}
	
	public Token getName(){
		return name;
	}
	
	public Map<Token, Value> getParameters(){
		return this.values;
	}
	
	@Override
	public void build(Builder builder) throws Exception{
		System.out.println("build annotation "+name+" | "+values);
		builder.append(name);
		if(values != null && values.size() > 0){
			builder.append("(");
			for(Entry<Token,Value> entry : values.entrySet()){
				if(entry.getKey() != null){
					builder.append(entry.getKey());
					builder.append("=");
				}
				entry.getValue().build(builder);
				builder.append(","); //replace by iterator splitter method
			}
			builder.append(")");
		}
	}

	@Override
	public <E extends JavaElement> E find(Function<JavaElement, Boolean> finder){
		for(Value value : this.values.values()){
			if(finder.apply(value)) return (E)value;
		}
		return null;
	}

	@Override
	public <E extends JavaElement> void findAll(Function<JavaElement, Boolean> finder, List<E> list){
		for(Value value : this.values.values()){
			if(finder.apply(value)) list.add((E)value);
		}
	}
}
